import 'dart:convert';
import 'dart:io';

import 'package:dashboard/constants/server_constants.dart';
import 'package:dashboard/exceptions/login_exception.dart';
import 'package:dashboard/util/user.dart';
import 'package:localstorage/localstorage.dart';
import 'package:logger/logger.dart';
import 'package:http/http.dart' as http;

class LoginHelper {
  static final LoginHelper _instance = LoginHelper._internal();
  factory LoginHelper() => _instance;

  // private constructor
  LoginHelper._internal();

  final Logger _logger = Logger();

  static const String _storageName = "login_storage";
  static const String _keyUser = "user";

  Future<LocalStorage> get _loginStorage async {
    LocalStorage storage = LocalStorage(_storageName);
    await storage.ready;
    return storage;
  }

  Future<BackendUser> get loggedInUser async {
    LocalStorage storage = await _loginStorage;

    Map<String, dynamic> row = storage.getItem(_keyUser);

    if (row == null) throw LoginException("No user is currently logged in");

    return BackendUser.fromMap(row);
  }

  Future<void> logout() async {
    LocalStorage storage = await _loginStorage;

    storage.setItem(_keyUser, null);
  }

  Future<BackendUser> login(String username, String password) async {
    if (username.isEmpty || password.isEmpty)
      throw LoginException("Fields cannot be empty!");

    String json = jsonEncode({
      "username": username,
      "password": password,
    });

    String url = "${ServerConstants.host}/api/tokens";

    _logger.v("calling $url");

    http.Response response = await http.post(
      url,
      headers: {
        HttpHeaders.contentTypeHeader: "application/json",
      },
      body: json,
    );

    if (response.statusCode != 200) {
      _logger.e(
          "Server responded with (${response.statusCode}): ${response.body}");

      if (response.statusCode == 400 &&
          response.body.endsWith("does not exist"))
        throw LoginException("No user was found!");

      throw LoginException(
          "An unexpected error occurred (${response.statusCode}");
    }
    _logger
        .v("Server responded with (${response.statusCode}): ${response.body}");

    Map<String, dynamic> data = jsonDecode(response.body);

    String token = data["token"];

    if (token == null || token.isEmpty)
      throw LoginException("An unknown problem occurred!");

    LocalStorage storage = await _loginStorage;
    BackendUser user = BackendUser(username, token);
    storage.setItem(_keyUser, user.toMap());

    Map<String, dynamic> row = storage.getItem(_keyUser);

    return BackendUser.fromMap(row);
  }
}
